library robomotion_app_lib;

import 'dart:convert';
import 'dart:io';

import 'package:flutter/widgets.dart';
import 'package:dio/dio.dart';

Dio _dio = Dio();

class Robomotion {
  final Config config;
  final Widget app;
  String _apiURL;

  Robomotion({@required this.config, @required this.app}) {
    _apiURL = _getAPIUrl();
  }

  void run() {
    _register().then((_) => runApp(this.app));
  }

  Future<void> _register() async {
    String url = "${this._apiURL}/v1/apps.register";
    Response<Map> resp = await _dio.post(url,
        options: Options(headers: {
          'Authorization': this.config._get("token"),
          'Content-Type': 'application/json; charset=UTF-8',
        }),
        data: {
          'repository_id': this.config._get('repository_id'),
          'namespace': this.config._get('namespace'),
          'url': this.config._get('url'),
        });

    if (!resp.data.containsKey("ok") || !resp.data["ok"]) {
      throw new Exception("Application registration failed: ${resp.data.toString()}");
    }
  }

  String _getAPIUrl() {
    String ws = this.config._get("workspace");
    List<String> parts = ws.split(".");
    String domain = parts.getRange(1, parts.length).join(".");
    String protocol = parts[0].split("//")[0];
    return domain == "robomotion.io" ? "$protocol//api.$domain" : "$ws/api";
  }
}

class Config {
  String path = "config.json";
  Map _data;

  Config({this.path}) {
    File(this.path).readAsString(encoding: Encoding.getByName("utf-8")).then((value) {
      _data = jsonDecode(value);
    });
  }

  dynamic _get(String key) {
    return _data[key];
  }
}
